from google.cloud import secretmanager

def access_secret_manager_secret(secretRef, project, version="latest"):
    """
    Access a secret from Google Secret Manager
    :param string secretRef: A Secret Manager secret name
    :param string project: GCP project name or project ID
    :param string version: Define a specific version to use, otherwise use latest
    :return: A plaintext secret value
    """ 
    client   = secretmanager.SecretManagerServiceClient()
    secret   = f"projects/{project}/secrets/{secretRef}/versions/{version}"

    try:
        response = client.access_secret_version(request={
            "name": secret
        })
    except Exception as e:
        return e

    return response.payload.data.decode("UTF-8")


from slack_sdk import WebClient
from slack_sdk.errors import SlackApiError

def send_slack_simple_message(slack_bot_token, channel, text, emoji=":robot_face:"):
    """
    Sends a simple Slack message.
    :param string slack_bot_token: A Slack bot token (xoxb-*) 
    :param string channel: Either channel name/ID or Slack UID
    :param string text: The message's text
    :param string emoji: Emoji to use as bot icon
    """
    client = WebClient(
        token=slack_bot_token
    )

    try:
        result = client.chat_postMessage(
            channel = f"{channel}",
            text=f"{text}",
            icon_emoji=f"{emoji}",
        )
    except SlackApiError as e:
        print(f"Error posting to Slack: {e}")



from prometheus_client import CollectorRegistry, Gauge, Counter, push_to_gateway

def push_metrics_pushgateway(metric_name,
metric_desc,
pushgateway_url,
value,
labels,
job="google-cloud-functions",
metric_type="gauge"):
  """
  Pushes a metric to a Prometheus PushGateway instance.
  :param string metric_name: Metric name to use
  :param string metric_desc: Metric description
  :param string pushgateway_url: A PushGateway URL in the form of http://pushgateway:9091
  :param float64 value: Metric value
  :param dict labels:
  :param string job:
  """
  registry = CollectorRegistry()

  _label_keys = []
  _label_values = []
  for key,val in labels.items():
      _label_keys.append(key)
      _label_values.append(val)

  if metric_type == 'gauge':
      g = Gauge(metric_name, metric_desc, _label_keys, registry=registry)
      g.labels(*_label_values).set(value)
  elif metric_type == 'counter':
      c = Counter(metric_name, metric_desc, _label_keys, registry=registry)
      c.labels(*_label_values).inc()

  try:
      push_to_gateway(pushgateway_url, job=job, registry=registry)
  except Exception as e:
      print(f"Error: {e}")
      return e



from google.cloud import storage

def push_object_to_gcs_bucket_from_stream(bytestream, fileName, bucketId, contentType="text/plain"):
    """
    Push a bucket object containing a bytestream
    Can be used for plaintext, ZIP archives, music files, anything that can be a bytestream
    :param object bytestream: File content
    :param string fileName: Created file name
    :param string bucketId: A GCP bucket ID
    :param string contentType: Content-Type header content https://www.iana.org/assignments/media-types/media-types.xhtml
    """
    client = storage.Client()
    bucket = client.get_bucket(bucketId)
    blob   = bucket.blob(fileName)
    
    try:
        blob.upload_from_string(bytestream, content_type=contentType)
    except Exception as e:
        return e

    return "OK"


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gcphelpers',
 'gcphelpers.prometheus',
 'gcphelpers.secretmanager',
 'gcphelpers.slack',
 'gcphelpers.storage']

package_data = \
{'': ['*']}

install_requires = \
['google-cloud-secret-manager>=2.7.2,<3.0.0',
 'google-cloud-storage>=1.42.3,<2.0.0',
 'pre-commit>=2.15.0,<3.0.0',
 'prometheus-client>=0.11.0,<0.12.0',
 'slack-sdk>=3.11.2,<4.0.0']

setup_kwargs = {
    'name': 'gcphelpers',
    'version': '0.0.11',
    'description': 'Helpers for GCP functions',
    'long_description': None,
    'author': 'Alex Orfanos',
    'author_email': 'alexandros@mailerlite.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
